/* 
 * File:   LCD_Display_MikroC.h
 * Author: paul_000
 *
 * Created on February 7, 2017, 9:11 PM
 */

#ifndef LCD_DISPLAY_MIKROC_H
#define	LCD_DISPLAY_MIKROC_H

#define _XTAL_FREQ 4000000
//#define LAT
#define DISPLAY_LINES   2
#define LCD_TIMEOUT 100     // max nr. of busy checks ...
#define LCD_CMD     0   // register selct
#define LCD_DATA    1
#define NOP         asm("nop")

#ifdef LAT
#define	LCD_DB	LATA
#define	LCD_D7	LATAbits.LATA3
#define	LCD_D6	LATAbits.LATA2
#define	LCD_D5	LATAbits.LATA1
#define	LCD_D4	LATAbits.LATA0
#define	LCD_EN	LATAbits.LATA5  //was LCD_E
//#define	LCD_RW	LATBbits.LATB5
#define	LCD_RS	LATAbits.LATA4
#else
#define	LCD_DB	PORTA
#define	LCD_D7	PORTAbits.RA3
#define	LCD_D6	PORTAbits.RA2
#define	LCD_D5	PORTAbits.RA1
#define	LCD_D4	PORTAbits.RA0
#define	LCD_EN	PORTAbits.RA5  //was LCD_E
//#define	LCD_RW	PORTAbits.LATB5
#define	LCD_RS	PORTAbits.RA4
#endif

#define	LCD_D7_DIR	TRISAbits.TRISA3
#define	LCD_D6_DIR	TRISAbits.TRISA2
#define	LCD_D5_DIR	TRISAbits.TRISA1
#define	LCD_D4_DIR	TRISAbits.TRISA0
#define	LCD_EN_DIR	TRISAbits.TRISA5  //was LCD_E
//#define	LCD_RW_Direction	TRISBbits.TRISB5
#define	LCD_RS_DIR	TRISAbits.TRISA4

#define LCD_DIR_OUT() LCD_D4_DIR = LCD_D5_DIR = LCD_D6_DIR = LCD_D7_DIR = 0
#define LCD_BUSY    1   // busy flag logic
#define LCD_STROBE()    NOP; LCD_EN = 1; __delay_us(10); LCD_EN = 0; __delay_us(50);
#define LCD_ENABLE()    LCD_EN = 1;
#define LCD_DISABLE()   LCD_EN = 0;

// Set CGRAM address            0   0   |   0   1   a   a   a   a   a   a
    #define CGRAM_ADDR          0x40
// Set DDRAM address            0   0   |   1   a   a   a   a   a   a   a
    #define CURSOR_ADDR         0x80


#define _LCD_FIRST_ROW          0x00
#define _LCD_SECOND_ROW         0x40
#define _LCD_THIRD_ROW          0x80
#define _LCD_FOURTH_ROW         0xc0

#define	_LCD_CLEAR              0b00000001
#define	_LCD_RETURN_HOME		0b00000010
#define _LCD_CURSOR_OFF         0b00000000
#define _LCD_UNDERLINE_ON
#define	_LCD_BLINK_CURSOR_ON	0b00000001
#define	_LCD_MOVE_CURSOR_LEFT	0x14
#define	_LCD_MOVE_CURSOR_RIGHT	0x14
#define	_LCD_TURN_ON			0b00000100
#define	_LCD_TURN_OFF           0b00001000
#define	_LCD_SHIFT_LEFT         0x18
#define	_LCD_SHIFT_RIGHT        0x1C
#define	_LCD_CURSOR_ON          0b00000010

//                              RS  R/W |   D7  D6  D5  D4  D3  D2  D1  D0
// Function Set                 0   0   |   0   0   1   DL  N   F   *   *
//      DL: 1 = 8-Bit   / 0 = 4-Bit interface
//      N:  1 = 2 lines / 0 = 1 line
//      F:  1 = 5x11    / 0 = 5x8 dots  (double hight bit in extended mode !!!)
    #define LCD_RESET           0x30
    #define FOUR_BIT            0x20  // 4-bit Interface
    #define FOUR_BIT_ONE_LINE   0x20
    #define FOUR_BIT_TWO_LINE   0x28
    #define EIGHT_BIT           0x30  // 8-bit Interface
    #define EIGHT_BIT_ONE_LINE  0x30
    #define EIGHT_BIT_TWO_LINE  0x38
// Entry mode set               0   0   |   0   0   0   0   0   1   I/D S
    #define ENTRY_MODE          0x04
    #define CURSOR_INC          0x02
    #define CURSOR_DEC          0x00
    #define DSHIFT_ON           0x01
    #define DSHIFT_OFF          0x00
// Display ctrl                 0   0   |   0   0   0   0   1   D   C   B
    #define DISPLAY_CTRL        0x08
    #define DISPLAY_ON          0x04
    #define DISPLAY_OFF         0x00
    #define CURSOR_ON           0x02
    #define CURSOR_OFF          0x00
    #define BLINK_ON            0x01
    #define BLINK_OFF           0x00
// Cursor / Display shift       0   0   |   0   0   0   1   S/C R/L -   -
    #define DISPLAY_SHIFT_R     0x1C
    #define DISPLAY_SHIFT_L     0x18
    #define CURSOR_MOVE_R       0x14
    #define CURSOR_MOVE_L       0x14

void    Lcd_Init();                        // Initialize LCD
void    Lcd_Cmd(char CMD);
void    Lcd_Data(char DATA);
void    Lcd_Write_Nybble(char BYTE);
void    Lcd_Out(char row, char col, far const char* msg);
void    Lcd_Chr(char row, char col, char out_char);
void    Lcd_Chr_Cp(char out_char);
//unsigned char LCD_Busy(void);

union LCDv8bit {
    char all;
    struct {
        unsigned LN : 4;
        unsigned HN : 4;
    };
    struct {
        unsigned bit0 : 1;
        unsigned bit1 : 1;
        unsigned bit2 : 1;
        unsigned bit3 : 1;
        unsigned bit4 : 1;
        unsigned bit5 : 1;
        unsigned bit6 : 1;
        unsigned bit7 : 1;
    };
};

#ifdef	__cplusplus
extern "C" {
#endif




#ifdef	__cplusplus
}
#endif

#endif	/* LCD_DISPLAY_MIKROC_H */

